/********************************

 Based on 

 Simple config file class.     

 Author: Ulrich Clanget
 Copying: Freeware

********************************/

/* ConfigGroup.cpp */

#include "config.h"
#include <qfile.h>
#include <iostream.h>
#include <qregexp.h>


ConfigGroup::ConfigGroup(const QString& groupname)
{
  setName(groupname);
}

ConfigGroup::ConfigGroup()
{
}

void ConfigGroup::setName(const QString& gname)
{
  // assign group name
  groupname = gname;
}

void ConfigGroup::writeEntry(const QString& key, const QString& value)
{
  // replace or insert new key
  map.replace(key, value);
}

const QString& ConfigGroup::readEntry(const QString& key, const QString& vdefault)
{
  // return value, if exists or default value
  const QString& tmp = map[key];
  
  return (tmp.isEmpty() ? vdefault : tmp);
}


/* ConfigFile.cpp */

//#include "ConfigFile.h"

//#include <qfile.h>
//#include <iostream.h>
//#include <qregexp.h>

#define MAX_LINE_LENGTH 1500

ConfigFile::ConfigFile(const QString& filename)
{
  setName(filename);

  // create default group
  setGroup("Default");
}

ConfigFile::ConfigFile()
{
}

ConfigFile::~ConfigFile()
{
  clear();
}

void ConfigFile::clear()
{
  ConfigMap::Iterator it;
  
  for(it = map.begin(); it != map.end(); ++it)
    delete it.data();
}

void ConfigFile::setName(const QString& filename)
{
  // assign file name
  fname = filename;
}

void ConfigFile::setGroup(const QString& group)
{
  if (map.find(group) == map.end())
    {
      // new group
      currentgroup = new ConfigGroup(group);
      //CHECK_PTR(currentgroup);

      map.insert(currentgroup->name(), currentgroup);
    }
  else currentgroup = map[group];
}

const QString& ConfigFile::group() const
{
  return currentgroup->name();
}

void ConfigFile::writeEntry(const QString& key, const QString& value)
{
  currentgroup->writeEntry(key, value);
}

void ConfigFile::writeEntry(const QString& key, int value)
{
  QString tmp;
  tmp.setNum(value);

  currentgroup->writeEntry(key, tmp);
}

void ConfigFile::writeEntry(const QString& key, double value)
{
  QString tmp;
  tmp.setNum(value);
  
  currentgroup->writeEntry(key, tmp);
}

const QString& ConfigFile::readEntry(const QString& key, const QString& default_value)
{
  return currentgroup->readEntry(key, default_value);
}

int ConfigFile::readEntry(const QString& key, int default_value)
{
  QString tmp;
  
  tmp = currentgroup->readEntry(key);
  if (tmp == QString::null) 
    return default_value;
  else 
    return tmp.toInt();
}

double ConfigFile::readEntry(const QString& key, double default_value)
{
  QString tmp;
  
  tmp = currentgroup->readEntry(key);
  if (tmp == QString::null) 
    return default_value;
  else 
    return tmp.toDouble();
}

void ConfigFile::write()
{
  if (fname.isEmpty()) return;

  // write to file
  QFile file(name());
  file.open(IO_WriteOnly);
  
  ConfigMap::Iterator it;
  
  for(it = map.begin(); it != map.end(); ++it)
    if (!it.data()->isEmpty())
      {
        // write group name
        file.writeBlock("[" + it.key() + "]\n", 3 + it.key().length());

        // write group data
        GroupMap::Iterator git;

        for (git = it.data()->map.begin(); git != it.data()->map.end(); ++git)
          file.writeBlock(git.key() + " = " + git.data() + "\n",
                          git.key().length() + 3 + git.data().length() + 1);
      }
 
  file.close();
}

void ConfigFile::read()
{
  if (fname.isEmpty()) return;
  
  // open file
  QFile file(name());
  file.open(IO_ReadOnly);
  
  QString line;

  while(file.readLine(line, MAX_LINE_LENGTH) != -1)
    { 
      // is it a group name?
      int index = line.find("[");

      if (index != -1)
        {
          // get group name
          int index2 = line.findRev("]");

          setGroup(line.mid(index + 1, index2 - index - 1));
        }
     
      // is it a key = value string
      if (index == -1)
      {
          // simplify white space
          line = line.stripWhiteSpace(); // tf
	  // remove all spaces from line
          index = line.find("=");
          
          if (index != -1)
          {
              	// get key and value
              	QString key = line.left(index);
		key=key.stripWhiteSpace();
              	//QString key = line.left(index - 1); //original
	  	// Check if there is a value
	  	QString value;
		if ((uint)index==(line.simplifyWhiteSpace().length()-1)) // no value
			value="";
		else
		{
              		value = line.right(line.length() - (index + 1)); // TF was +2
			value=value.stripWhiteSpace();
		}
              //QString value = line.right(line.length() - (index + 1)); // TF was +2

              writeEntry(key, value);
          }
      }
    }

  file.close();

  // switch to default group
  setGroup("Default");
}

QStringList ConfigFile::groupList()
{
  QStringList tmp;

  ConfigMap::Iterator it;
  
  for(it = map.begin(); it != map.end(); ++it)
    tmp.append(it.key());

  return tmp;
}

